# -*- coding: utf-8 -*-

EVENT_TYPE_EVENT = "event"
EVENT_TYPE_SPAN = "span"
EVENT_TYPE_STOP = "stop"
EVENT_TYPE_DESTROY = "destroy"

UOM_KEY = "uom"
VALUE_KEY = "value"
TIMESTAMP_KEY = "timestamp"

DEFAULT_ANNOTATION_DURATION = 30000
