# -*- coding:utf-8 -*-

ASSETS = "ASSETS"
MANAGE_WELLS = "MANAGE_WELLS"
DASHBOARDS = "DASHBOARDS"
USE_MESSENGER = "USE_MESSENGER"
MANAGE_DASHBOARDS = "MANAGE_DASHBOARDS"
MANAGE_ASSETS = "MANAGE_ASSETS"
EDIT_ANNOTATIONS = "EDIT_ANNOTATIONS"
MANAGE_PIPES_WIDGETS = "MANAGE_PIPES_WIDGETS"
VIEW_WELLS = "VIEW_WELLS"
MANAGE_MONITORING = "MANAGE_MONITORING"
CONSOLE = "CONSOLE"
CREATE_ROOMS = "CREATE_ROOMS"
IMPORT_DATA = "IMPORT_DATA"
ADMIN = "ADMIN"
ADVANCED_IMPORT_DATA = "ADVANCED_IMPORT_DATA"
